@@ -1,5 +1,3 @@
-from typing import Literal
-
 from .app import SkApp
 from .window import SkWindow
 
@@ -8,7 +6,14 @@ class SkAppWindow(SkWindow):
 
     _instance_count = 0
 
-    def __init__(self, *args, window_event_wait: bool = False, **kwargs) -> None:
+    def __init__(
+        self,
+        *args,
+        is_always_update: bool = ...,
+        is_get_context_on_focus: bool = ...,
+        vsync: bool = ...,
+        **kwargs,
+    ) -> None:
         """Main window that connects SkApp with SkWindow."""
         import platform
 
@@ -16,7 +21,11 @@ class SkAppWindow(SkWindow):
             if "force_hardware_acceleration" in kwargs.keys():
                 kwargs.pop("force_hardware_acceleration")
 
-        self.app = SkApp(window_event_wait=window_event_wait)
+        self.app = SkApp(
+            is_always_update=is_always_update,
+            is_get_context_on_focus=is_get_context_on_focus,
+            vsync=vsync,
+        )
         super().__init__(parent=self.app, *args, **kwargs)
         if self.__class__._instance_count == 0:
             self.__class__._instance_count += 1
