@@ -1,7 +1,9 @@
-import warnings
+import array
+import typing
 
 import skia
 
+from ..const import Orient
 from ..event import SkEvent
 
 
@@ -17,7 +19,7 @@ class SkContainer:
     guarantee the stability of inheriting SkContainer for third-party widgets.
 
     SkContainer class contains code for widget embedding, and layout handling, providing the
-    ability of containing `children` to widgets inerit from it. All other classes with such
+    ability of containing `children` to widgets inherit from it. All other classes with such
     abilities should be inherited from SkContainer.
 
     SkContainer has a `children` list, each item is a `SkWidget`, called `child`. This helps
@@ -47,49 +49,105 @@ class SkContainer:
 
     # region __init__ 初始化
 
+    parent: typing.Self
+    width: int | float
+    height: int | float
+
+    def __enter__(self):
+        self._handle_layout()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._handle_layout()
+        # 触发更新事件
+        self.event_trigger("update", SkEvent(event_type="update"))
+
     def __init__(self, allowed_out_of_bounds: bool = False):
 
         # self.parent = None
         self.children = []  # Children
 
-        from ..widgets.widget import SkWidget
+        from .widget import SkWidget
 
         self.draw_list: list[list[SkWidget]] = [
             [],  # Layout layer [SkWidget1, SkWidget2, ...]
             [],  # Floating layer [SkWidget1, SkWidget2, ...]
             [],  # Fixed layer [SkWidget1, SkWidget2, ...]
         ]
-        # self.layers_layout_type = ["none" for i in range(len(self.draw_list))]  # ['none', 'none', 'none']
+        self.layout_names = [None, None, None]
 
-        self._box_direction = None  # h(horizontal) or v(vertical)
-        self.allowed_out_of_bounds = allowed_out_of_bounds
+        # 【内部组件统计总占大小】
+        self.content_width = 0
+        self.content_height = 0
 
-        # self.bind("resize", self._handle_layout)
-        self.bind("resize", self._handle_layout)
-        self.bind("update", self._update)
+        # 【内部组件偏移，用于实现容器内部的滚动】
+        self._x_offset = 0
+        self._y_offset = 0
+        self.allowed_scrolled: bool = False
+        self.scroll_speed: float | int = 18  # 滚动距离：滚动量x滚动速度
 
-        if isinstance(self, SkWidget):
+        self._grid_lists = []  # [ [row1, ], [] ]
+        self._box_direction: Orient | None = None  # h(horizontal) or v(vertical)
+        self._flow_row = 0
+        self.allowed_out_of_bounds = (
+            allowed_out_of_bounds  # 【是否允许组件超出容器范围】
+        )
 
-            def children_resize(event: SkEvent):
-                for child in self.children:
-                    child.event_generate("resize", event)
-
-            self.bind("resize", children_resize)
-
-    def _update(self, event):
-        """Organize the layout and send an `update` event message to the child components
-
-        :param event: SkEvent
-        :return:
-        """
-        self._handle_layout(event)
-        for widget in self.children:
-            from suzaku.event import SkEvent
-
-            widget.event_generate("update", SkEvent(event_type="update"))
+        # Events
+        self.bind("resize", lambda _: self.update_layout())
 
     # endregion
 
+    def bind_scroll_event(self):
+        # 【容器绑定滚动事件，鼠标滚轮滚动可以滚动容器】
+        self.allowed_scrolled = True
+        self.window.bind("scroll", self.scroll_event)
+
+    def scroll_event(self, event: SkEvent) -> None:
+        """【处理滚动事件】"""
+        if self.allowed_scrolled:
+            if self.is_mouse_floating:
+                self.scroll(event.x_offset * 18, event.y_offset * 18)
+                return
+
+            for child in self.children:
+                if (
+                    child.is_mouse_floating
+                    and child.help_parent_scroll
+                    and child.parent == self
+                ):
+                    self.scroll(event.x_offset * 18, event.y_offset * 18)
+                    return
+
+    def update_scroll(self) -> None:
+        """【检查并更新滚动偏移量】"""
+        if self.content_width < self.width:
+            self._x_offset = 0
+        else:
+            self._x_offset = max(self.x_offset, -(self.content_width - self.width))
+        # 【防止容器超出下边界】
+        if self.content_height < self.height:
+            self._y_offset = 0
+        else:
+            self._y_offset = max(self.y_offset, -(self.content_height - self.height))
+
+    def scroll(
+        self,
+        x_offset: int | float,
+        y_offset: int | float,
+    ) -> None:
+        """【滚动容器】
+
+        :param x_offset: 【水平滚动量】
+        :param y_offset: 【垂直滚动量】
+        """
+        """if self._check_scroll(x_offset, y_offset):
+        self.y_offset = min(y_offset + self.y_offset, self.content_height)
+        """
+        self.x_offset = min(self.x_offset + x_offset, 0)
+        # 防止容器超出上边界
+        self.y_offset = min(self.y_offset + y_offset, 0)
+
     # region add_child 添加子元素
     def add_child(self, child):
         """Add child widget to window.
@@ -98,9 +156,39 @@ class SkContainer:
         """
         from .app import SkApp
 
-        if not isinstance(self.parent, SkApp):
-            self.parent.add_child(child)
-        self.children.append(child)
+        if not child in self.children:
+            if not isinstance(self.parent, SkApp):
+                self.parent.add_child(child)
+            self.children.append(child)
+
+    def remove_child(self, child):
+        """Remove child widget from window.
+        :param child: The child to remove"""
+        pass
+
+    def remove_all(self):
+        for child in self.children:
+            child.layout_forget()
+
+    def grid_map(self):
+        # Grid Map
+        from .widget import SkWidget
+
+        grid_map: list[list[SkWidget | None]] = []
+        children: list[SkWidget] = self.draw_list[0]
+
+        for child in children:
+            child_config = child.layout_config["grid"]
+            row, col = child_config["row"], child_config["column"]
+
+            if col > len(grid_map) - 1:
+                while col > len(grid_map) - 1:
+                    grid_map.append([])
+            if row > len(grid_map[col]) - 1:
+                while row > len(grid_map[col]) - 1:
+                    grid_map[col].append(None)
+            grid_map[col][row] = child
+        return grid_map
 
     def add_layout_child(self, child):
         """Add layout child widget to window.
@@ -113,19 +201,19 @@ class SkContainer:
         if "box" in layout_config:
             side = layout_config["box"]["side"]
             if side == "left" or side == "right":
-                direction = "h"
+                direction = Orient.H
             elif side == "top" or side == "bottom":
-                direction = "v"
+                direction = Orient.V
             else:
                 raise ValueError("Box layout side must be left, right, top or bottom.")
 
-            if self._box_direction == "v":
-                if direction == "h":
+            if self._box_direction == Orient.V:
+                if direction == Orient.H:
                     raise ValueError(
                         "Box layout can only be used with vertical direction."
                     )
-            elif self._box_direction == "h":
-                if direction == "v":
+            elif self._box_direction == Orient.H:
+                if direction == Orient.V:
                     raise ValueError(
                         "Box layout can only be used with horizontal direction."
                     )
@@ -133,7 +221,7 @@ class SkContainer:
                 self._box_direction = direction
 
         self.draw_list[0].append(child)
-        self._handle_layout()
+        self.update_layout()
 
     def add_floating_child(self, child):
         """Add floating child widget to window.
@@ -141,8 +229,9 @@ class SkContainer:
         :arg child: SkWidget
         :return: None
         """
+
         self.draw_list[1].append(child)
-        self._handle_layout()
+        self.update_layout()
 
     def add_fixed_child(self, child):
         """Add fixed child widget to window.
@@ -156,7 +245,7 @@ class SkContainer:
         :return: None
         """
         self.draw_list[2].append(child)
-        self._handle_layout()
+        self.update_layout()
 
     # endregion
 
@@ -183,10 +272,10 @@ class SkContainer:
                 canvas.save()
                 canvas.clipRect(
                     skia.Rect.MakeXYWH(
-                        x,
-                        y,
-                        self.width,
-                        self.height,
+                        x=x,
+                        y=y,
+                        w=self.width,
+                        h=self.height,
                     )
                 )
         for layer in self.draw_list:
@@ -199,10 +288,22 @@ class SkContainer:
 
     # region layout 布局
 
-    def update_layout(self):
+    def update_layout(self, event: SkEvent | None = None):
+        """if self.allowed_scrolled and self.y_offset < 0:
+        if not self._check_scroll(0, -5):
+            self._y_offset = self.height - self.content_height
+            if self._y_offset > 0:
+                self._y_offset = 0"""
+        self.update_scroll()
         self._handle_layout()
+        for widget in self.children:
+            widget.event_trigger("resize", SkEvent(event_type="resize"))
 
-    def _handle_layout(self, evt=None):
+    def record_content_size(self, child, padx=0, pady=0):
+        self.content_width = max(child.x + child.dwidth + padx, self.content_width)
+        self.content_height = max(child.y + child.dheight + pady, self.content_height)
+
+    def _handle_layout(self, event=None):
         """Handle layout of the container.
 
         :return: None
@@ -213,11 +314,23 @@ class SkContainer:
                     match child.layout_config:
                         case {"place": _}:
                             pass
+                        case {"grid": _}:
+                            self.layout_names[0] = "grid"
+                            self._handle_grid()
+                            break
                         case {"box": _}:
+                            self.layout_names[0] = "box"
                             self._handle_box()
                             break
                         case {"fixed": _}:
+                            self.layout_names[2] = "fixed"
                             self._handle_fixed(child)
+                        case {"flow": _}:
+                            self.layout_names[0] = "flow"
+                            self._handle_flow(child)
+
+    def _handle_flow(self, child):
+        pass
 
     def _handle_pack(self):
         pass
@@ -226,7 +339,46 @@ class SkContainer:
         pass
 
     def _handle_grid(self):
-        pass
+        from .widget import SkWidget
+
+        # Grid
+        col_heights: list[int | float] = []
+        row_widths: list[int | float] = []
+        # children: list[SkWidget] = self.draw_list[0]
+        grid_map = self.grid_map()
+
+        for col, cols in enumerate(grid_map):
+
+            for row, widget in enumerate(cols):
+                child_config = widget.layout_config["grid"]
+
+                left, top, right, bottom = self.unpack_padding(
+                    child_config["padx"],
+                    child_config["pady"],
+                )
+                if len(row_widths) <= row:
+                    row_widths.append(0)
+                row_widths[row] = max(row_widths[row], widget.dwidth)
+                if len(col_heights) <= col:
+                    col_heights.append(0)
+                col_heights[col] = max(col_heights[col], widget.dheight + left + bottom)
+            # print(row_widths, col_heights)
+            row_left = 0
+
+            for row, widget in enumerate(cols):
+                child_config = widget.layout_config["grid"]
+                # print(child_config)
+                col_top = sum(col_heights[:col])
+                left, top, right, bottom = self.unpack_padding(
+                    child_config["padx"],
+                    child_config["pady"],
+                )
+                widget.width, widget.height = (
+                    row_widths[row],
+                    col_heights[col] - top - bottom,
+                )
+                widget.x, widget.y = row_left + left, col_top + top
+                row_left = widget.x + widget.width + right
 
     def _handle_box(self) -> None:
         """Process box layout.
@@ -234,7 +386,9 @@ class SkContainer:
         :return: None
         """
 
-        from ..widgets.widget import SkWidget
+        # TODO 做好ipadx、ipady的处理
+
+        from .widget import SkWidget
 
         width = self.width  # container width
         height = self.height  # container height
@@ -259,7 +413,7 @@ class SkContainer:
                 fixed_children.append(child)
 
         # Horizontal Layout
-        if self._box_direction == "h":
+        if self._box_direction == Orient.H:
             # Calculate the width of the fixed children
             fixed_width: int | float = 0  # Occupied width of all fixed widgets enabled
             for fixed_child in fixed_children:
@@ -284,49 +438,42 @@ class SkContainer:
             # Left side
             last_child_left_x = 0
             for child in start_children:
-                if type(child.layout_config["box"]["padx"]) is tuple:
-                    left = child.layout_config["box"]["padx"][0]
-                    right = child.layout_config["box"]["padx"][1]
-                else:
-                    left = right = child.layout_config["box"]["padx"]
+                child_layout_config = child.layout_config["box"]
+                left, top, right, bottom = self.unpack_padding(
+                    child_layout_config["padx"],
+                    child_layout_config["pady"],
+                )
 
-                if type(child.layout_config["box"]["pady"]) is tuple:
-                    top = child.layout_config["box"]["pady"][0]
-                    bottom = child.layout_config["box"]["pady"][1]
-                else:
-                    top = bottom = child.layout_config["box"]["pady"]
-
-                if not child.layout_config["box"]["expand"]:
-                    child.width = child.cget("dheight")
+                child.width = width - left - right
+                if not child_layout_config["expand"]:
+                    child.width = child.dwidth
                 else:
                     child.width = expanded_width - left - right
                 child.height = height - top - bottom
                 child.x = last_child_left_x + left
-                child.y = top
+                child.y = top + self.y_offset
+                self.record_content_size(child, right, bottom)
                 last_child_left_x = child.x + child.width + right
+                child.x += self.x_offset
 
             # Right side
             last_child_right_x = width
             for child in end_children:
-                if type(child.layout_config["box"]["padx"]) is tuple:
-                    left = child.layout_config["box"]["padx"][0]
-                    right = child.layout_config["box"]["padx"][1]
-                else:
-                    left = right = child.layout_config["box"]["padx"]
+                child_layout_config = child.layout_config["box"]
+                left, top, right, bottom = self.unpack_padding(
+                    child_layout_config["padx"],
+                    child_layout_config["pady"],
+                )
 
-                if type(child.layout_config["box"]["pady"]) is tuple:
-                    top = child.layout_config["box"]["pady"][0]
-                    bottom = child.layout_config["box"]["pady"][1]
-                else:
-                    top = bottom = child.layout_config["box"]["pady"]
-
-                if not child.layout_config["box"]["expand"]:
-                    child.width = child.cget("dheight")
+                child.width = width - left - right
+                if not child_layout_config["expand"]:
+                    child.width = child.dwidth
                 else:
                     child.width = expanded_width - left - right
                 child.height = height - top - bottom
-                child.x = last_child_right_x - child.width - right
-                child.y = top
+                child.x = last_child_right_x - child.width - right + self.x_offset
+                child.y = top + self.y_offset
+                self.record_content_size(child, right, bottom)
                 last_child_right_x = last_child_right_x - child.width - left * 2
         else:  # Vertical Layout
             # Calculate the height of the fixed children
@@ -355,49 +502,38 @@ class SkContainer:
             last_child_bottom_y = 0  # Last bottom y position of the child component
             for child in start_children:  # Top side
                 child_layout_config = child.layout_config["box"]
-                if type(child_layout_config["padx"]) is tuple:
-                    left = child_layout_config["padx"][0]
-                    right = child_layout_config["padx"][1]
-                else:
-                    left = right = child_layout_config["padx"]
-
-                if type(child_layout_config["pady"]) is tuple:
-                    top = child_layout_config["pady"][0]
-                    bottom = child_layout_config["pady"][1]
-                else:
-                    top = bottom = child_layout_config["pady"]
+                left, top, right, bottom = self.unpack_padding(
+                    child.layout_config["box"]["padx"],
+                    child.layout_config["box"]["pady"],
+                )
 
                 child.width = width - left - right
                 if not child_layout_config["expand"]:
-                    child.height = child.cget("dheight")
+                    child.height = child.dheight
                 else:
                     child.height = expanded_height - top - bottom
-                child.x = left
+                child.x = left + self.x_offset
                 child.y = last_child_bottom_y + top
+                self.record_content_size(child, right, bottom)
                 last_child_bottom_y = child.y + child.height + bottom
+                child.y += self.y_offset
 
             last_child_top_y = height  # Last top y position of the child component
             for child in end_children:  # Bottom side
                 child_layout_config = child.layout_config["box"]
-                if type(child_layout_config["padx"]) is tuple:
-                    left = child_layout_config["padx"][0]
-                    right = child_layout_config["padx"][1]
-                else:
-                    left = right = child_layout_config["padx"]
-
-                if type(child_layout_config["pady"]) is tuple:
-                    top = child_layout_config["pady"][0]
-                    bottom = child_layout_config["pady"][1]
-                else:
-                    top = bottom = child_layout_config["pady"]
+                left, top, right, bottom = self.unpack_padding(
+                    child.layout_config["box"]["padx"],
+                    child.layout_config["box"]["pady"],
+                )
 
                 child.width = width - left - right
                 if not child_layout_config["expand"]:
-                    child.height = child.cget("dheight")
+                    child.height = child.dheight
                 else:
                     child.height = expanded_height - top - bottom
-                child.x = left
-                child.y = last_child_top_y - child.height - bottom
+                child.x = left + self.x_offset
+                child.y = last_child_top_y - child.height - bottom + self.x_offset
+                self.record_content_size(child, right, bottom)
                 last_child_top_y = last_child_top_y - child.height - top * 2
 
     def _handle_fixed(self, child):
@@ -405,21 +541,59 @@ class SkContainer:
 
         :param child: The child widget
         """
-        from ..widgets.window import SkWindow
+        config = child.layout_config["fixed"]
+        child.x = config["x"] + self.x_offset
+        child.y = config["y"] + self.y_offset
 
-        child.x = child.layout_config["fixed"]["x"]
-        child.y = child.layout_config["fixed"]["y"]
-        child.width = child.layout_config["fixed"]["width"]
-        child.height = child.layout_config["fixed"]["height"]
+        width = config["width"]
+        if not width:
+            width = child.dwidth
+
+        height = config["height"]
+        if not height:
+            height = child.dheight
+
+        child.width = width
+        child.height = height
 
     # endregion
 
     # region other 其他
-    def bind(self, *args, **kwargs):
-        raise RuntimeError(
-            "Anything inherited from SkContainer should support binding events!"
-            + "This error should be overrode by the actual bind function of "
-            + "SkWindow or SkWidget in normal cases."
-        )
+    @property
+    def visible_children(self):
+        children = []
+        for layer in self.draw_list:
+            for child in layer:
+                children.append(child)
+                if hasattr(child, "visible_children"):
+                    children.extend(child.visible_children)
+        return children
+
+    # endregion
+
+    # region Configure 属性配置
+    @property
+    def x_offset(self) -> int | float:
+        """
+        【x方向内部偏移，用于实现容器内部的滚动】
+        """
+        return self._x_offset
+
+    @x_offset.setter
+    def x_offset(self, value: int | float):
+        self._x_offset = value
+        self.update_layout(None)
+
+    @property
+    def y_offset(self) -> int | float:
+        """
+        【y方向内部偏移，用于实现容器内部的滚动】
+        """
+        return self._y_offset
+
+    @y_offset.setter
+    def y_offset(self, value: int | float):
+        self._y_offset = value
+        self.update_layout(None)
 
     # endregion
